#include "XY2RADec.h"

#include <Tracer.h>
#include <errstr.h>
#include "param.h"

#include "CalServer.h"
#include "Dal.h"

#include <iostream>
#include <vector>


#include "ecoordconv.h"

using namespace std;

XY2RADec::XY2RADec(double x, double y, string imagefile)
: _x(x), _y(y), _imagefile(imagefile)
{
	calEcoordConv();
}

void XY2RADec::calEcoordConv()
{
	// Convert X,Y Sky to RA,Dec by 'ecoordconv'
	// ecoordconv imageset=image.ds withcoords=yes coordtype=pos x=X y=Y
	//
	Info::Task* _ecoordconv = task("ecoordconv");
	ParameterInfo& ecoordconvpars = *_ecoordconv->parameters();
	ecoordconvpars.revertToDefault();
	ecoordconvpars["imageset"]->setValue(_imagefile);
	ecoordconvpars["withcoords"]->setValue(true);
	ecoordconvpars["coordtype"]->setValue("pos");
	ecoordconvpars["x"]->setValue(_x);
	ecoordconvpars["y"]->setValue(_y);

		try{
			task(ecoordconv);
			}
		catch (SasException::Error e) {
			errstr << _ecoordconv->name() << " terminated with the error " << e.code
			<< " that said: \n" << e.message
			<< Sas::warning("SubTaskError");
			}
		catch (SasException::Fatal e) {
			errstr << _ecoordconv->name() << " terminated with the fatal error " << e.code
			<< " that said: \n" << e.message
			<< Sas::warning("SubTaskError");
			}

		double ra  =  ecoordconvpars["ra_out"]->realValue();
		double dec =  ecoordconvpars["dec_out"]->realValue();

		_radec.push_back(ra);
		_radec.push_back(dec);

}


std::vector<double> XY2RADec::getCoord()
{
	return _radec;
}
