/*
 ebkgreg

   Authors:  Jose Vicente Perea - XMM-Newton SOC
         
   Description: EPIC background selection for Source Products extraction
 
   Comments: 
 
  $Id: module.cc,v 1.5 2000/10/02 08:30:13 gvacanti Exp $
*/

#include "ebkgreg.h"

#include <Tracer.h>
#include <errstr.h>
#include "param.h"

#include <unistd.h> // For getpid

#include "CalServer.h"
#include "Dal.h"

#include "DataFilter.h"
#include <iostream>
#include <cmath>
#include <vector>
#include <algorithm>
#include "esky2det.h"
#include "edet2sky.h"

#include "SasFileLocator.h"     //      for locating d/f color tables
#include <metatask.h>

#include "Bkgreg.h"
#include "Lookupradius.h"
#include "ProcSrcList.h"
#include "XY2RADec.h"

using namespace std;

void ebkgreg ()
{
PROC

/* Get the parameters from this command line */

    // Input image
	std::string imagefile = stringParameter("imageset");
	DataSet	*set = dataSetServer->open(imagefile,Dal::Read);

	// Get instrument and obs. mode from image file
	std::string instr;
	std::string obs_id;
	std::string datamode;
	std::string submode;

		if (set->hasAttribute("INSTRUME"))
			instr = set->attribute("INSTRUME")->asString();

		if (set->hasAttribute("OBS_ID") )
			obs_id = set->attribute("OBS_ID")->asString();

		if (set->hasAttribute("DATAMODE"))
			datamode = set->attribute("DATAMODE")->asString();

		if (set->hasAttribute("SUBMODE") )
			submode = set->attribute("SUBMODE")->asString();

	// Check image input
	if (datamode != "IMAGING"){
		errstr << "Input image not valid. Exiting."<< Sas::error("invalidImageType");
	}
	
	///////////////////////////////////
	//	Pixsize for Instrument PN :
	double pixsizePN = 4.18;
	//
	///////////////////////////////////
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	//		For Source List as input
	//
	if  (booleanParameter("withsrclist")) {

	// Read Source List file from command line option
	//
	string src_table = stringParameter("srclisttab");
	std::cout << "Processing input Source List : " << src_table << std::endl;

	/////////////////////////////////////////////////////////////////////////
	//	Get Best background position estimation
		
	ProcSrcList tempSrcList(src_table, imagefile, instr, submode, booleanParameter("withcoords"), booleanParameter("withsrcrad"), 0);
		
	/////////////////////////////////////////////////////////////////////////


	/////////////////////////////////////////////////////////////////////////
	//
	// Insert Bkg information in the initial input Source List
	//
	// Read Bkg info from temporal file:
		string temp_file = tempSrcList.getTempSrcList();
		DataSet * src_set = dataSetServer->open(temp_file,Dal::Read);
		Table * src_tab = src_set->table("SRCLIST");
		//unsigned src_N = src_tab->rows();

		  //ColumnData *src_num_ = src_tab->column("SRC_NUM")->data();
		  //int32 *src_num = src_num_->int32Data();
		  ColumnData *tab_bkg_detx = src_tab->column("DETX")->data();
		  real32 *bkg_detx = tab_bkg_detx->real32Data();
		  ColumnData *tab_bkg_dety = src_tab->column("DETY")->data();
		  real32 *bkg_dety = tab_bkg_dety->real32Data();

		  ColumnData *tab_bkg_ra = src_tab->column("RA")->data();
		  real64 *bkg_ra = tab_bkg_ra->real64Data();
		  ColumnData *tab_bkg_dec = src_tab->column("DEC")->data();
		  real64 *bkg_dec = tab_bkg_dec->real64Data();

		  ColumnData *tab_bkg_imx = src_tab->column("IMAGEX")->data();
		  real32 *bkg_imx = tab_bkg_imx->real32Data();
		  ColumnData *tab_bkg_imy = src_tab->column("IMAGEY")->data();
		  real32 *bkg_imy = tab_bkg_imy->real32Data();

		  ColumnData *tab_bkg_x = src_tab->column("X")->data();
		  real32 *bkg_x = tab_bkg_x->real32Data();
		  ColumnData *tab_bkg_y = src_tab->column("Y")->data();
		  real32 *bkg_y = tab_bkg_y->real32Data();

		  ColumnData *tab_bkg_rad = src_tab->column("BKG_RAD")->data();
		  real32 *bkg_rad___ = tab_bkg_rad->real32Data();

	//
	// Write Bkg info in initial Source List file:
		  DataSet *src_set_i = dataSetServer->open(src_table,Dal::Modify);
		  Table *src_tab_i = src_set_i->table("SRCLIST");
		  unsigned src_N_i = src_tab_i->rows();

		  real32 *bkg_detx_i  = src_tab_i->addColumn("BKG_DETX",Column::Real32,"Bkg Linearised Camera X-Coordinate o", "0.05 arcsec")->data()->real32Data();
		  real32 *bkg_dety_i  = src_tab_i->addColumn("BKG_DETY",Column::Real32,"Bkg Linearised Camera Y-Coordinate o", "0.05 arcsec")->data()->real32Data();

		  real32 *bkg_ra_i   = src_tab_i->addColumn("BKG_RA",Column::Real32,"Bkg Right-ascension", "deg")->data()->real32Data();
		  real32 *bkg_dec_i  = src_tab_i->addColumn("BKG_DEC",Column::Real32,"Bkg Declination", "deg")->data()->real32Data();

		  real32 *bkg_imx_i  = src_tab_i->addColumn("BKG_IMX",Column::Real32,"Bkg Image X", "pixel")->data()->real32Data();
		  real32 *bkg_imy_i  = src_tab_i->addColumn("BKG_IMY",Column::Real32,"Bkg Image Y", "pixel")->data()->real32Data();

		  real32 *bkg_x_i  = src_tab_i->addColumn("BKG_X",Column::Real32,"Bkg X", "0.05 arcsec")->data()->real32Data();
		  real32 *bkg_y_i  = src_tab_i->addColumn("BKG_Y",Column::Real32,"Bkg Y", "0.05 arcsec")->data()->real32Data();

		  real32 *bkg_rad_i  = src_tab_i->addColumn("BKG_RAD",Column::Real32,"Bkg Radius", "0.05 arcsec")->data()->real32Data();

		  //////////////////////////////////////////////////////////
		  // Populate columns :
				  for (unsigned long j = 0 ; j < src_N_i ; ++j){
					  bkg_detx_i[j] = bkg_detx[j];
					  bkg_dety_i[j] = bkg_dety[j];

					  bkg_ra_i[j] = bkg_ra[j];
					  bkg_dec_i[j] = bkg_dec[j];

					  bkg_imx_i[j] = bkg_imx[j];
					  bkg_imy_i[j] = bkg_imy[j];

					  bkg_x_i[j] = bkg_x[j];
					  bkg_y_i[j] = bkg_y[j];

					  bkg_rad_i[j] = (bkg_rad___[j] * pixsizePN) / 0.05;
				  }
			
			// double bkg_rad_phys = (bkg_rad[0] * pixsizePN) / 0.05;
			// double bkg_rad_arcsec = bkg_rad_phys * 0.05;
			
		  //////////////////////////////////////////////////////////

// ---- DEBUG ----
//cout << "SRC_NUM	  BkgDETX,DETY  	  BkgRA,Dec		  BkgIMX,IMY			  BkgX,Y" << endl;
//for (unsigned long i = 0 ; i < src_N ; ++i){
//for (unsigned long i = 0 ; i < 4 ; ++i){
// cout << src_num[i] << " - " << bkg_detx[i] << "," << bkg_dety[i] << "	" << bkg_ra[i] << "," << bkg_dec[i]
// << "	" << bkg_imx[i] << ","  << bkg_imy[i] << "		  " << bkg_x[i] << "," << bkg_y[i] << "		" << bkg_rad_i[i]<< endl;
// ---- DEBUG ----	  		  }

		  dataSetServer->close(src_set);
		  dataSetServer->close(src_set_i);


		  // Remove temporal file:
		  dataSetServer->remove(temp_file);

	}

//======================================================================================
//======================================================================================

	/////////////////////////////////////////////////////////////////////////////////////////////////
	//
	//		For Individual Source as input
	//


	// Individual source coordinates from the command-line:
	if  (booleanParameter("withcoords")) {
		std::cout << "No source list. Source Coordinates instead ..." << std::endl;

		string coordType = stringParameter("coordtype");

		double X = realParameter("x");
		double Y = realParameter("y");
		double ra;
		double dec;

		if (uppercase(coordType) == "POS"){
		// Input Coordinates "pos" => x,y = X,Y Sky
			// Convert X,Y Sky to RA,Dec by 'ecoordconv'
			// ecoordconv imageset=image.ds withcoords=yes coordtype=pos x=X y=Y
			cout << "Input coordinates are POS type. Converting to EQPOS (RA,Dec) ..." << endl;
			XY2RADec radec_(X, Y, imagefile);
			ra  = radec_.getCoord()[0];
			dec = radec_.getCoord()[1];

		} else if (uppercase(coordType) == "EQPOS") {
		// Input Coordinates "eqpos" => x,y = RA,Dec
			cout << "Input coordinates are already EQPOS type (RA,Dec)." << endl;
			//ra = realParameter("ra");
			//dec = realParameter("dec");
			ra  = X;
			dec = Y;
		} else {
            errstr << "Coordtype unknown: "<<coordType
             << Sas::error("invalidCoordType");
		}

		double rad = realParameter("r");	// Source extraction radius (rad) in arcsec

		cout << "Input Source position : RA, Dec, R (arcsec) = " << ra << ", " << dec << ", " << rad << endl;
		cout << "" << endl;

		/////////////////////////////////////////////////////////////////////////
		// Create Source List for individual source position:
		srand(getpid());
		string random_test = toString(rand());
		string outsrclistfile = "/tmp/individual_list" + random_test + ".fits";
		DataSet	*outsrcset = dataSetServer->open(outsrclistfile,Dal::Create);
		Table * outtabset = outsrcset->addTable("SRCLIST", 1); // Only 1 row

		// Insert RA,Dec in Source List:
		// Add Column to existing FITS table

				real64 *ra__  = outtabset->addColumn("RA",Column::Real64,"Right-ascension", "deg")->data()->real64Data();
				real64 *dec__ = outtabset->addColumn("DEC",Column::Real64,"Declination", "deg")->data()->real64Data();

				ra__[0]  = ra;
				dec__[0] = dec;

				dataSetServer->close(outsrcset);  // Close Table FITS file
				
		/////////////////////////////////////////////////////////////////////////
		//	Get Best background position estimation

		ProcSrcList tempSrcList(outsrclistfile, imagefile, instr, submode, booleanParameter("withcoords"), booleanParameter("withsrcrad"), rad);

		/////////////////////////////////////////////////////////////////////////

		// Printout results to stdout
		string temp_file = tempSrcList.getTempSrcList();
		DataSet * src_set_indiv =  dataSetServer->open(temp_file,Dal::Read);
		Table * src_tab_indiv = src_set_indiv->table("SRCLIST");

		ColumnData *bkg_ra_ = src_tab_indiv->column("RA")->data();
		real64 *bkg_ra = bkg_ra_->real64Data();
		ColumnData *bkg_dec_ = src_tab_indiv->column("DEC")->data();
		real64 *bkg_dec = bkg_dec_->real64Data();
		ColumnData *bkg_x_ = src_tab_indiv->column("X")->data();
		real32 *bkg_x = bkg_x_->real32Data();
		ColumnData *bkg_y_ = src_tab_indiv->column("Y")->data();
		real32 *bkg_y = bkg_y_->real32Data();

		ColumnData *bkg_rad_ = src_tab_indiv->column("BKG_RAD")->data();
		real32 *bkg_rad = bkg_rad_->real32Data();

		//double pixsizePN = 4.18;
		double bkg_rad_phys = (bkg_rad[0] * pixsizePN) / 0.05;
		double bkg_rad_arcsec = bkg_rad_phys * 0.05;

		cout << "#" << endl;
		cout << "# Instrument: " << instr << endl;
		cout << "#" << endl;
		cout << "# Source position RA, Dec  (deg.)  : " << ra << ", " << dec << endl;
		cout << "# Source extraction radius (arcsec): " << rad << endl;
		cout << "#" << endl;
		cout << "	Best background position estimation:" << endl;

		if (bkg_rad_arcsec >= 0){
		cout << "	  RA, Dec (deg.)             : " << bkg_ra[0] << ", " << bkg_dec[0] << endl;
		cout << "	  Extraction radius (arcsec) : " << bkg_rad_arcsec << endl;
		cout << "" << endl;
		cout << "	  X,Y Sky Coord.                     : " << bkg_x[0] << ", " << bkg_y[0] << endl;
		cout << "	  Extraction radius (X,Y Sky Coord.) : " << bkg_rad_phys << endl;
		cout << "" << endl;
		} else {
			cout << "	** There is no space in CCD for background selection for this source." << endl;
			  cout << "		Extraction radius : " << bkg_rad_arcsec << endl;
			    cout << "" << endl;
		}
		// Set output parameters
		setParameter("bkg_ra_out",bkg_ra[0]);
		setParameter("bkg_dec_out",bkg_dec[0]);
		setParameter("bkg_rad_arcsec_out",bkg_rad_arcsec);
		setParameter("bkg_x_out",bkg_x[0]);
		setParameter("bkg_y_out",bkg_y[0]);
		setParameter("bkg_rad_phys_out",bkg_rad_phys);


		// Remove temporal files:
		dataSetServer->remove(outsrclistfile);
		dataSetServer->remove(temp_file);

	}


ENDPROC
}
